@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import argparse
 import os
+import re
 import subprocess
 import tempfile
 from dataclasses import dataclass, replace
@@ -587,28 +588,53 @@ class CompilationSetting:
     def preprocess_program(
         self,
         program: SourceProgram,
+        make_compiler_agnostic: bool = False,
         additional_flags: tuple[str, ...] = tuple(),
         timeout: Optional[int] = None,
-    ) -> CompilationInfo:
+    ) -> SourceProgram:
         """Preprocesses the program
 
         Args:
            program (SourceProgram): input program
            additional_flags (tuple[str, ...]): additional flags used for the compilation
+           make_compiler_agnostic (bool):
+               if true will try to remove certain constructs (e.g., attributes)
+               such that the resulting program can be compiled with both gcc and clang
            timeout (int | None): timeout in seconds for the compilation command
 
         Returns:
-            CompilationInfo:
-                information about the compilation, the
-                preprocessing result is stored in the output
+            Source:
+                the prepocessed program
 
         """
 
-        return self.compile_program(
+        preprocessed_source = self.compile_program(
             program,
             CompilationOutput(Path(""), CompilationOutputKind.Unspecified),
             ("-P", "-E") + additional_flags,
             timeout=timeout,
+        ).stdout_stderr_output
+
+        if make_compiler_agnostic:
+            # remove malloc attributes with args, clang doesn't understand these
+            preprocessed_source = re.sub(
+                r"__attribute__ \(\(__malloc__ \(.*, .*\)\)\)", r"", preprocessed_source
+            )
+            # remove f128 builtins builtins, clang doesn't understand these
+            preprocessed_source = re.sub(
+                r"extern int [^;]*f128[^;]*;", r"", preprocessed_source
+            )
+            # remove Float*** typedefs, gcc doesn't like these
+            preprocessed_source = re.sub(
+                r"typedef [^;]*_Float\d+x?;", r"", preprocessed_source
+            )
+
+        return replace(
+            program,
+            code=preprocessed_source,
+            defined_macros=tuple(),
+            include_paths=tuple(),
+            system_include_paths=tuple(),
         )
 
 
@@ -797,20 +823,25 @@ class CComp:
         return CComp(exe=Path(ccomp).resolve(strict=True))
 
     def check_program(
-        self, program: SourceProgram, timeout: Optional[int] = None
+        self, program: SourceProgram, timeout: Optional[int] = None, debug: bool = False
     ) -> bool:
         """Checks the input program for errors using ccomp's interpreter mode.
 
         Args:
            program (SourceProgram): the input program
            timeout (int | None): timeout in seconds for the checking
+           debug (bool): if true ccomp's output will be printed on failure
 
         Returns:
             bool:
                 was the check successful?
         """
         assert program.language == Language.C
-        with TemporaryFile(contents=program.code, suffix=".c") as tf:
+
+        # ccomp doesn't like these
+        code = re.sub(r"__asm__ [^\)]*\)", r"", program.code)
+
+        with TemporaryFile(contents=code, suffix=".c") as tf:
             cmd = (
                 [
                     str(self.exe),
@@ -832,7 +863,9 @@ class CComp:
                     additional_env={"TMPDIR": str(tempfile.gettempdir())},
                     timeout=timeout,
                 )
-            except subprocess.CalledProcessError:
+            except subprocess.CalledProcessError as e:
+                if debug:
+                    print(CompileError.from_called_process_exception(" ".join(cmd), e))
                 return False
             return True
 
